/* JSON parser.
 *
 * This parser makes use of the ECMAScript lexer machinery.  It
 * handles more input than the RFC requires, but should be correct for
 * correct input (modulo obscure Unicode issues, maybe).
 */

use default namespace internal;

var scan;
var token = Token::NONE;

ESC function parseJSON(input, filename="**stdin**") {
    scan = new Lex::Scanner(input, filename);
    next();
    if (hd() == Token::LeftBrace || hd() == Token::LeftBracket)
        return datum();
    fail("Illegal datum: must be array or object literal.");
}

function datum() {
    switch (hd()) {
    case Token::True:          return next(), true;
    case Token::False:         return next(), false;
    case Token::Null:          return next(), null;
    case Token::StringLiteral: return let (s = lexeme()) next(), s;
    case Token::DoubleLiteral: return let (n = parseFloat(lexeme())) next(), n;
    case Token::IntLiteral:    return let (n = parseInt(lexeme())) next(), n; 
    case Token::UIntLiteral:   return let (n = parseInt(lexeme())) next(), n; 
    case Token::LeftBracket:   return arrayLiteral();
    case Token::LeftBrace:     return objectLiteral();
    default:                   fail("Unknown token: " + Token::tokenText(hd()));
    }
}

function arrayLiteral() {
    var a = new Array;
    eat(Token::LeftBracket);
    if (hd() != Token::RightBracket) {
        do 
            a.push(datum());
        while (match(Token::Comma));
    }
    eat(Token::RightBracket);
    return a;
}

function objectLiteral() {
    var o = new Object;
    eat(Token::LeftBrace);
    if (hd() != Token::RightBrace) {
        do {
            let name = datum();
            if (!(datum is String))
                fail("Field name must be a string");
            eat(Token::Colon);
            let value = datum();
            o[name] = value;
        } while (match(Token::Comma));
    }
    eat(Token::RightBrace);
    return o;
}

function hd()
    Token::tokenStore[token].kind;

function lexeme()
    Token::tokenStore[token].text;

function next() {
    token = scan.start();
}

function eat(t) {
    if (hd() != t)
        fail("Expected " + Token::tokenText(t) + " but found " + Token::tokenText(hd()));
    next();
}

function match(t) {
    if (hd() == t) {
        next();
        return true;
    }
    return false;
}

function fail(m) {
    throw new Error(m);
}

